#!/usr/bin/perl -w

# $Id: mail-out.pl,v 1.2.2.3 2006/05/26 23:04:50 killes Exp $

# THIS IS PROTOTYPE CODE AND SHOULD NOT BE USED IN PRODUCTION SYSTEMS!

# To quickly read the Perldoc below on a Unix system, use
#
# $ pod2man <FILE> | man -l -

=pod

=head1 NAME

mail-out.pl - outgoing mail handler for og2list.module

=head1 Synopsis

C<mail-in.pl [-d]>

=head1 Description

mail-out.pl reads messages fed into the og2list outgoing queue and
sends them out via a locelly installed MTA's /usr/bin/sendmail
interface.

=head1 Unfinished work, TODO's

=item ...

=head1 License

mail-out.pl, a helper script to send mail queued from the og2list
Drupal module.

Copyright (C) 2006 Hilko Bengen

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

=cut

use strict;

use FindBin;
use Mail::Verp;
Mail::Verp->separator('+');
use MIME::Entity;
# Used for encoding the subject (if necessary)
use Encode qw(encode);
use DBI;

use vars qw(
	    $dsn $dbh

	    $get_content_stmt
	    $get_rcpt_stmt
	    $set_sentflag_stmt
	    $get_sender_stmt

	    $get_content_handle
	    $get_rcpt_handle
	    $set_sentflag_handle
	    $get_sender_handle

	    $dbtype $dbuser $dbauth $dbhost $dbase
	    $mail_domain
	    $sendmail_binary @sendmail_args
	   );

use vars qw(
	    $mid $from_name $from_address $to_name $to_address
	    $subject $message_id $content_type $body
	   );

# ------------------------------------------------------------------------

# Default values
$sendmail_binary = "/usr/lib/sendmail";
@sendmail_args = ("-oi","-oem","-f");
$mail_domain = "localhost";

$dbtype = "mysql";
$dbase  = "";
$dbhost = "localhost";
$dbuser = "";
$dbauth = "";

do "$FindBin::Bin/mail.conf";

# ------------------------------------------------------------------------

$get_content_stmt =<<EOF;
SELECT DISTINCT c.mid,
                c.from_name,
                c.from_address,
                c.to_name,
                c.to_address,
                c.subject,
                c.msgid,
                c.content_type,
                c.body
FROM og2list_outgoing_recipients AS r,
     og2list_outgoing_content AS c
WHERE (r.mid = c.mid) AND (r.status = 1)
EOF

$get_rcpt_stmt =<<EOF;
SELECT recipient_address
FROM og2list_outgoing_recipients
WHERE (mid = ?) AND (status = 1)
EOF

$get_sender_stmt=<<EOF;
SELECT DISTINCT g.recipient
FROM og2list_outgoing_recipients AS r, og2list_groups AS g
WHERE (r.nid = g.nid) AND (r.mid = ?);
EOF

$set_sentflag_stmt=<<EOF;
UPDATE og2list_outgoing_recipients
SET status = 2
WHERE (mid = ?) AND (recipient_address = ?)
EOF

# ----------------------------------------------------------------------

# Make database connection
$dsn = "DBI:$dbtype:$dbase:$dbhost";
$dbh = DBI->connect($dsn, $dbuser, $dbauth) or die;

# Set up queries
$get_content_handle = $dbh->prepare($get_content_stmt) or die;
$get_rcpt_handle = $dbh->prepare($get_rcpt_stmt) or die;
$set_sentflag_handle = $dbh->prepare($set_sentflag_stmt) or die;
$get_sender_handle = $dbh->prepare($get_sender_stmt) or die;

# Get messages that are to be processed.
$get_content_handle->execute();

while ( ($mid, $from_name, $from_address, $to_name, $to_address,
	 $subject, $message_id, $content_type, $body) =
	$get_content_handle->fetchrow_array() ) {
  my $msg = MIME::Entity->build
    ("From"       => "$from_name <$from_address>",
     "To"         => "$to_name <$to_address\@$mail_domain>",
     "Subject"    => encode('MIME-Header', "$subject"),
     "Type"       => "$content_type",
     "Encoding"   => "-SUGGEST",
     "Charset"    => "utf-8",
     "Message-ID" => "$message_id",
     "Data"       => "<html><body>$body</body></html>",
     "X-Mailer"   => undef);

  my $localpart;
  my $rcpt_addr;

  $get_rcpt_handle->execute($mid);
  $get_sender_handle->execute($mid);
  ($localpart) = $get_sender_handle->fetchrow_array();

  print "Sending Message $mid to\n";
  while ( ($rcpt_addr) = $get_rcpt_handle->fetchrow_array() ) {
    my $verp_addr = Mail::Verp->encode("$localpart\@$mail_domain", $rcpt_addr);
    print "  sender=$verp_addr rcpt=$rcpt_addr\n";

    open(SENDMAIL, '|-') || exec($sendmail_binary, @sendmail_args,
 				 $verp_addr, $rcpt_addr);
    $msg->print(\*SENDMAIL);
    close(SENDMAIL) && do {
      $set_sentflag_handle->execute($mid, $rcpt_addr);
    };
  }
}

# Local Variables:
# mode:cperl
# End:
